﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using static System.Console;
//using static System.Diagnostics.Debug;

namespace CSharp6_0Exp
{
    class Program
    {
        static void Main(string[] args)
        {
            NullConditionalOperators();
            //AutoPropertyInitializers();
            //GetterOnlyAutoProperties();
            //NameofExpressions();
            //ExpressionBodiedFunctionsAndProperties();
            //StringInterpolation();
            //UsingStatic();
            //IndexInitializers();
            //ExceptionFilters();
            //AwaitInCatchAndFinallyBlocks();

            Console.WriteLine("Press any key to exit...");
            Console.ReadKey();
        }

        private static void NullConditionalOperators()
        {
            // Ziel
            //Console.WriteLine(
            //    UserGroups.All[0].Events.First().Title
            //    );

            // Vorher
            if (UserGroups.All != null)
            {
                if (UserGroups.All[0] != null && UserGroups.All[0].Events != null)
                {
                    if (UserGroups.All[0].Events.FirstOrDefault() != null)
                    {
                        Console.WriteLine(UserGroups.All[0].Events.First().Title);
                    }
                }
            }

            // C# 6.0
            Console.WriteLine(
                UserGroups.All?[0]?.Events?.FirstOrDefault()?.Title
                );
        }

        private static void AutoPropertyInitializers()
        {
            var inDotNet = new UserGroup() { Name = "INdotNET" };
            // Die Liste Events wurde bereits initialisiert!
            inDotNet.Events.Add(new Event() { Title = "C# 6.0" });

            Console.WriteLine(inDotNet.Events.Count);
        }

        private static void GetterOnlyAutoProperties()
        {
            var inDotNet = new UserGroup() { Name = "INdotNET" };
            Console.WriteLine(inDotNet.BestProgrammingLanguage);

            //inDotNet.BestProgrammingLanguage = "FoxPro"; // nicht erlaubt, weil readonly!
        }

        private static void NameofExpressions()
        {
            var userGroupEvent = new Event() { Title = "C# 6.0" };
            userGroupEvent.Invite(null); //-> ArgumentNullException
        }

        private static void ExpressionBodiedFunctionsAndProperties()
        {
            var userGroup = new UserGroup() { Name = "INdotNET" };
            Console.WriteLine(userGroup.CalculateBestTimeForUsingCSharp6()); // Rückmeldung aus Lamda-Zuweisung
        }

        private static void StringInterpolation()
        {
            int cSharpVersion = 6;
            string strAlt = string.Format("Hier geht es um C# {0}", cSharpVersion);

            string strNeu = $"Hier geht es um C# {cSharpVersion}";

            System.Threading.Thread.CurrentThread.CurrentCulture = new System.Globalization.CultureInfo("en-US");
            string strFormatted = $"Hier geht es um C# {cSharpVersion:0.0}";

            Console.WriteLine(strAlt);
            Console.WriteLine(strNeu);
            Console.WriteLine(strFormatted);
        }

        private static void UsingStatic()
        {
            Console.WriteLine("Ohne using static System.Console");

            WriteLine("Mit using static System.Console");
        }

        private static void IndexInitializers()
        {
            var dictAlt = new SortedDictionary<int, string>()
            {
                { 2, "dot" },
                //{ 2, "dot" },
                { 3, "NET" },
                { 1, "IN" },
            };

            foreach (var entry in dictAlt)
            {
                Console.WriteLine(entry.Value);
            }

            var dictNeu = new SortedDictionary<int, string>()
            {
                [2] = "dot",
                //[2] = "dot",
                [3] = "NET",
                [1] = "IN",
            };

            foreach (var entry in dictNeu)
            {
                Console.WriteLine(entry.Value);
            }
        }

        private static void ExceptionFilters()
        {
            // Vorher:
            try
            {
                var userGroupEvent = new Event() { Title = "C# 6.0" };
                userGroupEvent.Invite(null);
            }
            catch (ArgumentNullException ex)
            {
                if (ex.ParamName == "attendees")
                {
                    Console.Error.WriteLine("Ein null Parameter bedeutet natürlich ALLE .NET-Entwickler!");
                }
                else
                {
                    Console.Error.WriteLine("Welche Argumente denn noch?");
                }
            }

            // C# 6.0
            try
            {
                var userGroupEvent = new Event() { Title = "C# 6.0" };
                userGroupEvent.Invite(null);
            }
            catch (ArgumentNullException ex) when (ex.ParamName == "attendees")
            {
                Console.Error.WriteLine("Ein null Parameter bedeutet natürlich ALLE .NET-Entwickler!");
            }
            catch (ArgumentNullException)
            {
                Console.Error.WriteLine("Welche Argumente denn noch?");
            }
        }

        private static async void AwaitInCatchAndFinallyBlocks()
        {
            // C# 6.0
            try
            {
                //System.IO.File.ReadAllText(@"C:\Datei_existiert_nicht.txt"); // Achtung: Statische File-Methoden nicht asynchron!
                using (var streamReader = new System.IO.StreamReader(@"C:\Datei_existiert_nicht.txt"))
                {
                    string text = await streamReader.ReadToEndAsync();
                    Console.WriteLine(text);
                }
            }
            catch (Exception ex)
            {
                await Console.Error.WriteLineAsync("Async Error:");
                await Console.Error.WriteLineAsync(ex.ToString());
            }
            finally
            {
                await Task.Delay(TimeSpan.FromSeconds(1));
                await Console.Error.WriteLineAsync("1 Sekunde lang aufgeräumt!");
            }
        }
    }
}